@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * Copyright © 2000 Keith Packard
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -90,7 +88,7 @@ XftDrawDepth (XftDraw *draw)
 	Window		    root;
 	int		    x, y;
 	unsigned int	    width, height, borderWidth, depth;
-	if (XGetGeometry (draw->dpy, draw->drawable, 
+	if (XGetGeometry (draw->dpy, draw->drawable,
 			  &root, &x, &y, &width, &height,
 			  &borderWidth, &depth))
 	    draw->depth = depth;
@@ -106,7 +104,7 @@ XftDrawBitsPerPixel (XftDraw	*draw)
 	XPixmapFormatValues *formats;
 	int		    nformats;
 	unsigned int	    depth;
-	
+
 	if ((depth = XftDrawDepth (draw)) &&
 	    (formats = XListPixmapFormats (draw->dpy, &nformats)))
 	{
@@ -136,8 +134,8 @@ XftDrawCreate (Display   *dpy,
 
     draw = (XftDraw *) malloc (sizeof (XftDraw));
     if (!draw)
-	return 0;
-    
+	return NULL;
+
     draw->dpy = dpy;
     draw->drawable = drawable;
     draw->screen = _XftDrawScreen (dpy, drawable, visual);
@@ -146,7 +144,7 @@ XftDrawCreate (Display   *dpy,
     draw->visual = visual;
     draw->colormap = colormap;
     draw->render.pict = 0;
-    draw->core.gc = 0;
+    draw->core.gc = NULL;
     draw->core.use_pixmap = 0;
     draw->clip_type = XftClipTypeNone;
     draw->subwindow_mode = ClipByChildren;
@@ -162,16 +160,16 @@ XftDrawCreateBitmap (Display	*dpy,
 
     draw = (XftDraw *) malloc (sizeof (XftDraw));
     if (!draw)
-	return 0;
+	return NULL;
     draw->dpy = dpy;
     draw->drawable = (Drawable) bitmap;
-    draw->screen = _XftDrawScreen (dpy, bitmap, 0);
+    draw->screen = _XftDrawScreen (dpy, bitmap, NULL);
     draw->depth = 1;
     draw->bits_per_pixel = 1;
-    draw->visual = 0;
+    draw->visual = NULL;
     draw->colormap = 0;
     draw->render.pict = 0;
-    draw->core.gc = 0;
+    draw->core.gc = NULL;
     draw->core.use_pixmap = 0;
     draw->clip_type = XftClipTypeNone;
     draw->subwindow_mode = ClipByChildren;
@@ -188,16 +186,16 @@ XftDrawCreateAlpha (Display *dpy,
 
     draw = (XftDraw *) malloc (sizeof (XftDraw));
     if (!draw)
-	return 0;
+	return NULL;
     draw->dpy = dpy;
     draw->drawable = (Drawable) pixmap;
-    draw->screen = _XftDrawScreen (dpy, pixmap, 0);
+    draw->screen = _XftDrawScreen (dpy, pixmap, NULL);
     draw->depth = depth;
     draw->bits_per_pixel = 0;	/* don't find out until we need it */
-    draw->visual = 0;
+    draw->visual = NULL;
     draw->colormap = 0;
     draw->render.pict = 0;
-    draw->core.gc = 0;
+    draw->core.gc = NULL;
     draw->core.use_pixmap = 0;
     draw->clip_type = XftClipTypeNone;
     draw->subwindow_mode = ClipByChildren;
@@ -211,9 +209,9 @@ _XftDrawFormat (XftDraw	*draw)
     XftDisplayInfo  *info = _XftDisplayInfoGet (draw->dpy, True);
 
     if (!info || !info->hasRender)
-	return 0;
+	return NULL;
 
-    if (draw->visual == 0)
+    if (draw->visual == NULL)
     {
 	XRenderPictFormat   pf;
 
@@ -246,7 +244,7 @@ XftDrawChange (XftDraw	*draw,
     if (draw->core.gc)
     {
 	XFreeGC (draw->dpy, draw->core.gc);
-	draw->core.gc = 0;
+	draw->core.gc = NULL;
     }
 }
 
@@ -303,9 +301,9 @@ XftDrawSrcPicture (XftDraw *draw, _Xconst XftColor *color)
     int		    i;
     XftColor	    bitmapColor;
 
-    if (!info)
+    if (!info || !info->solidFormat)
 	return 0;
-    
+
     /*
      * Monochrome targets require special handling; the PictOp controls
      * the color, and the color must be opaque
@@ -324,9 +322,9 @@ XftDrawSrcPicture (XftDraw *draw, _Xconst XftColor *color)
      */
     for (i = 0; i < XFT_NUM_SOLID_COLOR; i++)
     {
-	if (info->colors[i].pict && 
+	if (info->colors[i].pict &&
 	    info->colors[i].screen == draw->screen &&
-	    !memcmp ((void *) &color->color, 
+	    !memcmp ((void *) &color->color,
 		     (void *) &info->colors[i].color,
 		     sizeof (XRenderColor)))
 	    return info->colors[i].pict;
@@ -350,7 +348,7 @@ XftDrawSrcPicture (XftDraw *draw, _Xconst XftColor *color)
     {
 	Pixmap			    pix;
         XRenderPictureAttributes    pa;
-	
+
 	pix = XCreatePixmap (dpy, RootWindow (dpy, draw->screen), 1, 1,
 			     info->solidFormat->depth);
 	pa.repeat = True;
@@ -393,7 +391,7 @@ _XftDrawRenderPrepare (XftDraw	*draw)
 	format = _XftDrawFormat (draw);
 	if (!format)
 	    return FcFalse;
-	
+
 	if (draw->subwindow_mode == IncludeInferiors)
 	{
 	    pa.subwindow_mode = IncludeInferiors;
@@ -456,7 +454,7 @@ _XftDrawCorePrepare (XftDraw *draw, _Xconst XftColor *color)
     XSetForeground (draw->dpy, draw->core.gc, color->pixel);
     return FcTrue;
 }
-			
+
 _X_EXPORT Picture
 XftDrawPicture (XftDraw *draw)
 {
@@ -481,7 +479,7 @@ XftDrawGlyphs (XftDraw		*draw,
     if (font->format)
     {
 	Picture	    src;
-	
+
 	if (_XftDrawRenderPrepare (draw) &&
 	    (src = XftDrawSrcPicture (draw, color)))
 	    XftGlyphRender (draw->dpy, _XftDrawOp (draw, color),
@@ -499,7 +497,7 @@ _X_EXPORT void
 XftDrawString8 (XftDraw		    *draw,
 		_Xconst XftColor    *color,
 		XftFont		    *pub,
-		int		    x, 
+		int		    x,
 		int		    y,
 		_Xconst FcChar8	    *string,
 		int		    len)
@@ -509,7 +507,7 @@ XftDrawString8 (XftDraw		    *draw,
 
     if (XftDebug () & XFT_DBG_DRAW)
 	printf ("DrawString \"%*.*s\"\n", len, len, string);
-    
+
     if (len <= NUM_LOCAL)
 	glyphs = glyphs_local;
     else
@@ -547,7 +545,7 @@ XftDrawString16 (XftDraw	    *draw,
     }
     for (i = 0; i < len; i++)
 	glyphs[i] = XftCharIndex (draw->dpy, pub, string[i]);
-    
+
     XftDrawGlyphs (draw, color, pub, x, y, glyphs, len);
     if (glyphs != glyphs_local)
 	free (glyphs);
@@ -575,7 +573,7 @@ XftDrawString32 (XftDraw	    *draw,
     }
     for (i = 0; i < len; i++)
 	glyphs[i] = XftCharIndex (draw->dpy, pub, string[i]);
-    
+
     XftDrawGlyphs (draw, color, pub, x, y, glyphs, len);
     if (glyphs != glyphs_local)
 	free (glyphs);
@@ -585,7 +583,7 @@ _X_EXPORT void
 XftDrawStringUtf8 (XftDraw	    *draw,
 		   _Xconst XftColor *color,
 		   XftFont	    *pub,
-		   int		    x, 
+		   int		    x,
 		   int		    y,
 		   _Xconst FcChar8  *string,
 		   int		    len)
@@ -797,7 +795,7 @@ XftDrawCharFontSpec (XftDraw			*draw,
 _X_EXPORT void
 XftDrawRect (XftDraw		*draw,
 	     _Xconst XftColor	*color,
-	     int		x, 
+	     int		x,
 	     int		y,
 	     unsigned int	width,
 	     unsigned int	height)
@@ -809,7 +807,10 @@ XftDrawRect (XftDraw		*draw,
     }
     else if (_XftDrawCorePrepare (draw, color))
     {
-	XftRectCore (draw, color, x, y, width, height);
+	/* note: not XftRectCore() */
+	XSetForeground (draw->dpy, draw->core.gc, color->pixel);
+	XFillRectangle (draw->dpy, draw->drawable, draw->core.gc,
+			x, y, width, height);
     }
 }
 
@@ -817,16 +818,16 @@ _X_EXPORT Bool
 XftDrawSetClip (XftDraw	*draw,
 		Region	r)
 {
-    Region			n = 0;
+    Region			n = NULL;
 
     /*
      * Check for quick exits
      */
     if (!r && draw->clip_type == XftClipTypeNone)
 	return True;
-    
-    if (r && 
-	draw->clip_type == XftClipTypeRegion && 
+
+    if (r &&
+	draw->clip_type == XftClipTypeRegion &&
 	XEqualRegion (r, draw->clip.region))
     {
 	return True;
@@ -861,7 +862,7 @@ XftDrawSetClip (XftDraw	*draw,
     case XftClipTypeNone:
 	break;
     }
-    
+
     /*
      * Set the clip
      */
@@ -906,7 +907,7 @@ XftDrawSetClipRectangles (XftDraw		*draw,
 			  _Xconst XRectangle	*rects,
 			  int			n)
 {
-    XftClipRect	*new = 0;
+    XftClipRect	*new = NULL;
 
     /*
      * Check for quick exit
@@ -945,7 +946,7 @@ XftDrawSetClipRectangles (XftDraw		*draw,
     case XftClipTypeNone:
 	break;
     }
-    
+
     /*
      * Set the clip
      */
@@ -985,7 +986,7 @@ XftDrawSetSubwindowMode (XftDraw *draw, int mode)
 	XRenderPictureAttributes    pa;
 
 	pa.subwindow_mode = mode;
-	XRenderChangePicture (draw->dpy, draw->render.pict, 
+	XRenderChangePicture (draw->dpy, draw->render.pict,
 			      CPSubwindowMode, &pa);
     }
     if (draw->core.gc)
